

/*
    This is the csound_main.c from csound vers. 5.16 frontends,
    but adapted to use the API call for csoundScoreExtract.

    You would run something like the C::B project CscoreAPI
    to generate a .sco, create an extract file, then run
    this project to extract and create a new .sco file.

    NOTE: csound is not initialized here since the csound library
    is not utilized.  Extract is a standalone .exe in csound which
    can be used from the command line.  The only thing happening here
    is we're invoking extract from the csoundAPI instead of the
    command line.

*/

/* Console Csound using the Csound API. */
#include "csound.h"
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <errno.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#ifdef GNU_GETTEXT
#include <locale.h>
#endif

#ifdef LINUX
extern int set_rt_priority(int argc, char **argv);
#endif

static FILE *logFile = NULL;

static void msg_callback(CSOUND *csound,
                         int attr, const char *format, va_list args)
{
    (void) csound;
    if ((attr & CSOUNDMSG_TYPE_MASK) != CSOUNDMSG_REALTIME) {
      vfprintf(logFile, format, args);
      fflush(logFile);
     }
    #if defined(WIN32) || defined(MAC)
    switch (attr & CSOUNDMSG_TYPE_MASK) {
        case CSOUNDMSG_ERROR:
        case CSOUNDMSG_WARNING:
        case CSOUNDMSG_REALTIME:
        break;
      default:
        vfprintf(logFile, format, args);
        return;
    }
    #endif

    vfprintf(stderr, format, args);
}

static void nomsg_callback(CSOUND *csound,
  int attr, const char *format, va_list args){ return; }

int main(int argc, char **argv)
{
    CSOUND  *csound;
    char    *fname = NULL;
    int     i, result, nomessages=0;
    FILE *inFile, *outFile, *extractFile;
    const char *csvector[] = {"testin.sco", "extractout.sco", "xfile"};

#ifdef GNU_GETTEXT
    const char* lang;
#endif

    /* set stdout to non buffering if not outputing to console window */
    if (!isatty(fileno(stdout))) {
#if !defined(WIN32)
      setvbuf(stdout, (char*) NULL, _IONBF, 0);
#endif
    }

#ifdef GNU_GETTEXT
    /* We need to set the locale for the translations to work */
    lang = csoundGetEnv(NULL, "CS_LANG");
    /* If set, use that. Otherwise use the system locale */
    if(lang == NULL)
        lang = setlocale(LC_MESSAGES, "");
    else
        lang = setlocale(LC_MESSAGES, lang);
    /* Should we warn if we couldn't set the locale (lang == NULL)? */
    /* If the strings for this binary are ever translated,
     * the textdomain should be set here */
#endif

    /* Real-time scheduling on Linux by Istvan Varga (Jan 6 2002) */
#ifdef LINUX
    if (set_rt_priority(argc, argv) != 0)
      return -1;

#endif
    /* open log file if specified */
    for (i = 1; i < argc; i++) {
      if (strncmp(argv[i], "-O", 2) == 0 && (int) strlen(argv[i]) > 2)
        fname = argv[i] + 2;
      else if (strncmp(argv[i], "--logfile=", 10) == 0 &&
               (int) strlen(argv[i]) > 10)
        fname = argv[i] + 10;
      else if (i < (argc - 1) && strcmp(argv[i], "-O") == 0)
        fname = argv[i + 1];
    }
    if (fname != NULL) {
      if (!strcmp(fname, "NULL") || !strcmp(fname, "null"))
               nomessages = 1;
      else if ((logFile = fopen(fname, "w")) == NULL) {
        fprintf(stderr, "Error opening log file '%s': %s\n",
                        fname, strerror(errno));
        return -1;
      }
    }

    /* initialise Csound library */
    // not using the csound library here for extract
    //csoundInitialize(&argc, &argv, 0);

    /*  Create Csound. */
    csound = csoundCreate(NULL);

    /* if logging to file, set message callback */
    if (logFile != NULL)
      csoundSetMessageCallback(csound, msg_callback);
    else if (nomessages)
      csoundSetMessageCallback(csound, nomsg_callback);

    /**
     * Reset and prepare an instance of Csound for compilation.
     * Returns CSOUND_SUCCESS on success, and CSOUND_ERROR or
     * CSOUND_MEMORY if an error occured.
     */
    result = csoundPreCompile(csound);

    if (result < 0)
    {
        fprintf(stderr, "Could not precompile csound.\n");
        return -1;
    }

    //assigns FILE ptr to file and opens it for "r", or "w"
    if (!(inFile = fopen(csvector[0], "r")))
    {
        fprintf(stderr, "Cannot open the input file '%s'\n", csvector[0]);
        return -1;
    }
    if (!(outFile = fopen(csvector[1], "w")))
    {
        fprintf(stderr, "Cannot open the output file '%s'\n", csvector[1]);
        return -1;
    }
    if (!(extractFile = fopen(csvector[2], "r")))
    {
        fprintf(stderr, "Cannot open the extract file '%s'\n", csvector[2]);
        return -1;
    }

// standalone extract.exe commands
//		extract <in >out
//	./extract xfile <jstest.sco >jstextract.sco

     /**
     * Extracts from 'inFile', controlled by 'extractFile', and writes
     * the result to 'outFile'. The Csound instance should be initialised
     * with csoundPreCompile() before calling this function, and csoundReset()
     * should be called after score extraction to clean up.
     * The return value is zero on success.
     */
    //PUBLIC int csoundScoreExtract(CSOUND *,
           // FILE *inFile, FILE *outFile, FILE *extractFile);

    csoundScoreExtract(csound, inFile, outFile, extractFile );

//    /*  One complete performance cycle. */
//    result = csoundCompile(csound, argc, argv);
//    if (!result)
//      result = csoundPerform(csound);


    fclose(inFile); //may not be needed
    fclose(outFile); //may not be needed
    fclose(extractFile); //may not be needed

    csoundCleanup(csound); //may not be needed
    csoundReset(csound);

    /* delete Csound instance */
     csoundDestroy(csound);
    /* close log file */
    if (logFile != NULL)
      fclose(logFile);
#if 0
    /* remove global configuration variables, if there are any */
    csoundDeleteAllGlobalConfigurationVariables();
#endif
    return (result >= 0 ? 0 : result);
}
